// Header File
// Created 11/21/2010; 6:59:19 AM

//******************************************************************************
//
// Project:         Jumpman 89
//
// file name:       jm_enemies.h
//
// author:          David Randall "Ranman" 
//                  derandall99@gmail.com
//
// notes::          Jumpman was originally created by Randy Glover for the
//                  Atari 400, Commodore 64, IBM PCjr, and Apple II in 1982
//
//******************************************************************************


#ifndef _jm_enemies_h
#define _jm_enemies_h

#include "BasicTypes.h"

#include "jm_enemies.h"
#include "jm_objects.h"
#include "jm_jumpman.h"

#define ENEMY_STOP_SPEED 0
#define ENEMY_MOVE_SPEED 2
#define ENEMY_COUNT_MAX  24


typedef enum
{
	AI_TYPE_None                        = 0,
	AI_TYPE_Still                       = 1,
	AI_TYPE_Falling_Explode             = 2,
	AI_TYPE_Flying_Random               = 3,
	AI_TYPE_Flying_Waypoints            = 4,
	AI_TYPE_Flying_Tracking             = 5,
	AI_TYPE_Flying_Up_Tracking          = 6,
	AI_TYPE_Walking_Random              = 7,
	AI_TYPE_Walking_Waypoints           = 8,
	AI_TYPE_Walking_Tracking            = 9,
	AI_TYPE_Walking_Tracking_Waypoints  = 10,
	AI_TYPE_Total
} AI_TYPE_Type;


#define WAYPOINT_COUNT_MAX     32


typedef struct
{
	uint2 enemy_type;       // GAME_PIECE_Type
	OBJECT_Type* object;
	int2  x;                // current lcd x coord
	int2  y;                // current lcd y coord
	int2  delay;            // frame delay count
	int2  counter;          // generic counter
	int2  move_history_index;          // generic counter

	uint1 sprite_index[5][4];
	uint1 sprite_width[5];
	uint1 anim_step_max[5]; // animation step for zig-zag and hailstones
	union
	{
		ucoord1 waypoint_list_pos[WAYPOINT_COUNT_MAX];
		uvect1  waypoint_list_vec[WAYPOINT_COUNT_MAX];
	};

  uint1 waypoint_index;
  uint1 waypoint_total;
  int2  waypoint_move_step;
  
	uint1 anim_step;        // animation step for zig-zag and hailstones
	uint1 height;          // generic counter
	uint1 height_max;          // generic counter
	uint1 speed;        // animation step for zig-zag and hailstones
	uint1 direction;        // DIR_Type
	uint1 direction_count;        // DIR_Type
	uint1 ai_type;
	uint1 bullet_fire_count;
  uint1 waypoint_command;
  uint1 misc;

	bool  hold;
	bool  reset;
	bool  reset_on_death;
	bool  valid;

//	uint1 bullet_index[4];
} ENEMY_OBJECT_Type;

unsigned char door[]=
{
  0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,
  0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,
  0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,
  0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,0x0f,
  //0xf0,0xf0,0xf0,0xf0,0xf0,0xf0,0xf0,0xf0,
  //0xf0,0xf0,0xf0,0xf0,0xf0,0xf0,0xf0,0xf0,
  //0xf0,0xf0,0xf0,0xf0,0xf0,0xf0,0xf0,0xf0,
  //0xf0,0xf0,0xf0,0xf0,0xf0,0xf0,0xf0,0xf0,
};

/*
typedef struct
{
	uint2 enemy_type;       // GAME_PIECE_Type
	OBJECT_Type* object;
	int2  x;                // current lcd x coord
	int2  y;                // current lcd y coord
	int2  delay;            // frame delay count
	int2  counter;          // generic counter

	uint1 sprite_index[5][4];
	uint1 sprite_width[5];
	uint1 anim_step_max[5]; // animation step for zig-zag and hailstones

	uint1 anim_step;        // animation step for zig-zag and hailstones
	uint1 speed;            // speed of enemy;
	uint1 direction;        // DIR_Type
	uint1 ai_type;
//	uint1 script_id;

	bool  hold;
	bool  reset;
//	uint1 bullet_index[4];
} ENEMY_OBJECT_Type;
*/

extern ENEMY_OBJECT_Type enemy_list[ENEMY_COUNT_MAX];
extern int2 enemy_count;


extern const ENEMY_OBJECT_Type enemy_definition_list[];


void DrawEnemies(void);
void UpdateEnemies(void);
void CheckEnemies(void);
void InitEnemies(void);
void ResetEnemy(ENEMY_OBJECT_Type* enemy);
void ExtinguishAllFires(void);
void ReleaseNextEnemy(uint1 enemy_type);
void Teleport(int2 enemy_object_id);
void ReleaseHatchling(int2 enemy_object_id);
void ResetEnemiesOnDeath(void);
void VerticalAdjustEnemy(ENEMY_OBJECT_Type* enemy);
void CheckMoveEnemy(ENEMY_OBJECT_Type* enemy);
ENEMY_OBJECT_Type* FindEnemyObject(int2 enemy_object_id);


#endif
