// C SourceFile
// Created 6/20/2010; 12:21:50 AM

//******************************************************************************
//
// Project:         Jumpman 89
//
// file name:       jm_jumpman.c
//
// author:          David Randall "Ranman" 
//                  derandall99@gmail.com
//
// notes::          Jumpman was originally created by Randy Glover for the
//                  Atari 400, Commodore 64, IBM PCjr, and Apple II in 1982
//
//******************************************************************************


#include <tigcclib.h>
#include "extgraph.h"

#include "BasicTypes.h"
#include "GrayUtils.h"
#include "FontUtils.h"
#include "TimeUtils.h"
#include "BasicUtils.h"
#include "FileUtils.h"

#include "jm_jumpman.h"
#include "jm_graphics.h"
#include "jm_collectables.h"
#include "jm_enemies.h"
#include "jm_bullets.h"
#include "jm_platforms.h"
#include "jm_moving_platforms.h"
#include "jm_levels.h"
#include "jm_objects.h"
#include "jm_titlescreen.h"
#include "jm_scores.h"
#include "jm_timers.h"

// default game speed is 3
#define DEFAULT_SPEED (3)
// default sleep time is the sleep time for speed 3
#define DEFAULT_SLEEP ((DEFAULT_SPEED + 1) * 5)
#define BONUS_POINT_DECREMENT_VALUE (25)


//#define FAST_START

// Global Variables
bool jumpman_dead;
//bool jump_in_progress;
int2 jumpmen_remaining;

MOVE_History_Type move_history[MOVE_HISTORY_MAX];
int2 move_history_index;

//
SPRITE_Type jumpman;


const ANIM_Type action_anim_sequence[ACTION_Total] =
{
	{16,{ 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1},
	    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
		  { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},}, // ACTION_None,
	{16,{ 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2},
	    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
	    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},}, // ACTION_Ladder_Still,
	{ 2,{ 6, 8, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
	    { 2, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
	    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},}, // ACTION_Walk,
	{15,{ 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0},
	    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
	    {-3,-2,-2,-1,-1, 0, 0, 0, 1, 1, 2, 2, 3, 3, 3, 0},}, // ACTION_Jump_Up,
	{16,{10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10},
	    { 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2},
	    {-3,-2,-2,-1,-1, 0, 0, 0, 1, 1, 2, 2, 3, 3, 3, 3},}, // ACTION_Jump,
	{ 4,{ 2, 3, 4, 5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
	    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
	    {-2,-2,-2,-2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},}, // ACTION_Ladder,
	{ 2,{12,13, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
	    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
	    {-2,-2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},}, // ACTION_Chain,
	{ 2,{12,13, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
	    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
	    { 2, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},}, // ACTION_Rope,
	{ 1,{14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
	    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
	    { 3, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},}, // ACTION_Fall,
	{12,{15,16,17,16,15,14,15,16,17,16,15,14, 0, 0, 0, 0},
	    { 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 0, 0, 0, 0},
	    {-2,-2,-2,-1,-1, 0, 0, 1, 1, 2, 2, 2, 0, 0, 0, 0},}, // ACTION_Bounce,
	{ 4,{18,18,19,19, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
	    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
	    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},}, // ACTION_Dead,
	{ 1,{ 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
	    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
	    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},}, // ACTION_Shoot,
};

// Main Function
void _main(void)
{
	bool done = false;
	int2 choice = 0;

	exit_game = false;

	pSprites1 = (uint1*)j3;
	pSprites2 = (uint1*)j7;

	// Allocate necessary space
	external_level_buffer = malloc(EXTERNAL_LEVEL_BUFFER_SIZE + 4 * LCD_SIZE + plane_offset_92plus);
	if(!external_level_buffer) return;
	
	plane_visible_0 = external_level_buffer + EXTERNAL_LEVEL_BUFFER_SIZE;
	//if(!plane_visible_0) return;
	plane_visible_1 = plane_visible_0 + LCD_SIZE;
	//if(!plane_visible_1) return;
	plane_active_0  = plane_visible_1 + LCD_SIZE;
	//if(!plane_active_0) return;
	plane_active_1  = plane_active_0  + LCD_SIZE;
	//if(!plane_active_1) return;

/*
92+
14*30+5
*/
	//plane_visible_0 = &visible_plane_0[0]; // 8*30
	//plane_visible_1 = &visible_plane_1[0]; // 8*30
	//plane_active_0  = &active_plane_0[0]; // 8*30
	//plane_active_1  = &active_plane_1[0]; // 8*30

	jumpman_dead = true;
	game_score = 0;

//	LoadLevelPack();
	ReadHighScores();

	// we are entering the game for the first time
	// so wait for the Enter key to be released.
	while (_keytest (RR_ENTER));

	// define the font we want to use
	//SetUserFont(font4x6 - 32 * 6, 4, 6);
	SetUserFont(font4x7 - 32 * 7, 4, 7);

	// turn on single buffered grayscale graphics
	GrayScaleOn(true);
//	GrayScaleOn(false);

	// seed random number generator
	randomize();

	// initialize the autoquit counter
	ResetAutoQuit();

	while(!done)
	{
#ifdef FAST_START
		choice = 1; //der added to speed up debugging
#else
		choice = DrawTitleScreen(); //der removed to speed up debugging
#endif

		if (exit_game)
		{
			done = true;
		}
		else if (choice == 1)
		{
			PlayGame();
		}
//		else if (choice == 2)
//		{
//			DrawAboutScreen();
//		}
	}

	GrayScaleOff();
	
	
	free (external_level_buffer);
	//free (plane_visible_0);
	//free (plane_visible_1);
	//free (plane_active_0);
	//free (plane_active_1);
	
}

void DrawStatusBar(void)
{
	char buffer[48];

	//sprintf(buffer, "JUMPMEN:%02d (%03d,%02d) B:%04d SCORE:%07lu", jumpmen_remaining, jumpman.x, jumpman.y, bonus_points_level, game_score);
	sprintf(buffer, "JUMPMEN:%02d L:%02d S:%d B:%04d SCORE:%07lu", jumpmen_remaining, current_level.level_number, game_speed, bonus_points_level, game_score);
	GrayUserDrawStr(0,92, buffer, COLOR_White, 0x00);
}

void SpawnJumpman(void)
{
	OBJECT_Type* object = FindObjectEntryByType(GAME_PIECE_Jumpman, true);

	jumpman_dead = false;

	jumpman.special_ability = object->special_ability;
	jumpman.x = object->x * 4;
	jumpman.y = object->y;
#if 1
	jumpman.x_orig = jumpman.x;
	jumpman.y_orig = jumpman.y;
	jumpman.anim_step_cur = 0;
	jumpman.anim_step_max = action_anim_sequence[0].anim_step_max;
	jumpman.direction = DIR_None;
	jumpman.action = ACTION_None;
	jumpman.bullet_index[0] = 0;
	jumpman.bullet_index[1] = 0;
	jumpman.bullet_index[2] = 0;
	jumpman.bullet_index[3] = 0;
	jumpman.visible = true;
#else
	SetJumpmanAction(ACTION_None, DIR_None);
#endif

	InitBullets();

	move_history_index = 0;
	//InitEnemies();
}

void PlayGame(void)
{
	bool  done = false;
	bool  action_key_pressed;
	uint4 sleep_time = 0;

	titlescreen    = false;
	jumpman_dead   = true;
	reset_game     = false;
	level_complete = false;
	frame_count    = 0;

	level_number       = 1;
	game_score         = 0;
	bonus_points_total = 0;
	jumpmen_remaining  = 10; //der
	game_speed         = DEFAULT_SPEED;

	EraseScreen();
	ResetAutoQuit();
	SetGameStage(STAGE_load_level_file);

	while(!done)
	{
		CheckAutoQuit(GAME_AUTOTIMER);
		CheckExitGame();
		CheckResetGame();

		if (!scores_inserted)
			CheckAdjustContrast();

		sleep_time = (game_speed - 1) * 5;

		action_key_pressed = (_keytest (RR_2ND));

		switch (game_stage)
		{
			case STAGE_load_level_file:
				DrawSelectScreen();
				break;

			case STAGE_init_level:

				EraseScreen();
				InitLevel();
				AdjustGameSpeed();
				SetGameStage(STAGE_display_level);

				break;

			case STAGE_display_level:

				DisplayNextLevel();
				AdjustGameSpeed();

				sleep_time = DEFAULT_SLEEP;

#ifdef FAST_START
				SetGameStage(STAGE_spawn_jumpman); //der added to speed up debugging & screenshots
#else
				if ((game_stage_count > 40) || (_keytest (RR_2ND)))
				{
					SetGameStage(STAGE_spawn_jumpman);
				}
#endif
				break;

			case STAGE_spawn_jumpman:

				DrawPlatformLevel();
				DrawMovingPlatforms();
				SpawnJumpman();
				UpdateJumpman();
				AdjustGameSpeed();

				//if (game_stage_count > 0)
				{
					SetGameStage(STAGE_play_game);
				}

				break;

			case STAGE_play_game:

				DrawPlatformLevel();

				UpdateCollectables();
				DrawCollectables();

				UpdateEnemies();
				DrawEnemies();

				UpdateMovingPlatforms();
				DrawMovingPlatforms();

				//der CheckKeyPress();
				UpdateJumpman();
				UpdateJumpmanMoveHistory();
				CheckKeyPress();

				UpdateBullets();
				DrawBullets();

				UpdateTimers();
				DrawTimers();
				
				CheckTimers();
				CheckBullets();
				CheckCollectables();
				CheckEnemies();
				CheckMovingPlatforms();

				DrawStatusBar();
				DrawBonusPoints();
				DrawLetters();

//der Quit level early for debugging purposes
//der if (_keytest (RR_X))level_complete = true; //der

				if (level_complete)
				{
					jumpman.x_freeze = jumpman.x;
					jumpman.y_freeze = jumpman.y;
					jumpman.sprite_index_freeze = jumpman.sprite_index;

					SetGameStage(STAGE_level_complete);
				}
				else if (jumpman_dead)
				{
					SetGameStage(STAGE_dead_jumpman);
				}

				if (bonus_points_level >= 100)
				{
					if ((game_stage_count % 100) == 99)
						bonus_points_level -= 100;
				}

				break;

			case STAGE_dead_jumpman:

				if (jumpmen_remaining > 1)
				{
					SetGameStage(STAGE_spawn_jumpman);
					ResetEnemiesOnDeath();
					DrawLetters();
				}
				else
				{
					EraseScreen();
					SetGameStage(STAGE_game_over);
				}

				jumpmen_remaining--;

				break;

			case STAGE_level_complete:

				level_complete = false;
				sleep_time = DEFAULT_SLEEP;

				GrayFillScreen(COLOR_Black);

				DrawPlatformLevel();
				DrawCollectables();
				DrawEnemies();
				DrawJumpman(jumpman.sprite_index_freeze, jumpman.x_freeze, jumpman.y_freeze);
				DrawBullets();
				DrawStatusBar();
				DrawBonusPoints();
				DrawLetters();

				if (action_key_pressed)
				{
					game_stage_count += 4;
				}

				if (bonus_points_level > 0)
				{
					if (action_key_pressed)
					{
						bonus_points_total += bonus_points_level;
						game_score += bonus_points_level;
						bonus_points_level = 0;
					}
					else
					{
						bonus_points_total += BONUS_POINT_DECREMENT_VALUE;
						game_score += BONUS_POINT_DECREMENT_VALUE;
						bonus_points_level -= BONUS_POINT_DECREMENT_VALUE;
					}

					if (bonus_points_level == 0)
						game_stage_count = 0;
				}
				else if (game_stage_count > 50)
				{
					if (current_level.final_level)
					{
						bonus_points_jumpman = jumpmen_remaining * 500;
						SetGameStage(STAGE_game_over);
					}
					else
					{
						level_number++;
						SetGameStage(STAGE_init_level);
					}
				}

				break;

			case STAGE_game_over:
			{
				char buffer[48];
				static bool bp_complete;
				static bool bp_complete_count;
				int2 x_pos = 0;

				GrayFillScreen(COLOR_Black);
				//DrawGameOver();
				//DrawStatusBar();
				DrawFinalScores();

				sprintf(buffer, "JUMPMEN BONUS: %02d * 500 pts/each = %05u", jumpmen_remaining, bonus_points_jumpman);

				if (action_key_pressed)
				{
					if (game_stage_count < 150)
						game_stage_count += 24;
					else
						game_stage_count += 4;
				}

				if (game_stage_count < 24)
				{
					buffer[13] = 0;
					x_pos = (26 - 0) * 2;
				}
				else if (game_stage_count < 50)
				{
					buffer[13] = 0;
					x_pos = (26 - (game_stage_count - 24)) * 2;
				}
				else if (game_stage_count < 75)
				{
					buffer[17] = 0;
				}
				else if (game_stage_count < 100)
				{
					buffer[19] = 0;
				}
				else if (game_stage_count < 125)
				{
					buffer[34] = 0;
				}
				else if (game_stage_count < 150)
				{
					bp_complete = false;
				}
				else if (!bp_complete)
				{
					if (bonus_points_jumpman > 0)
					{
						if (action_key_pressed)
						{
							bonus_points_total += bonus_points_jumpman;
							game_score += bonus_points_jumpman;
							bonus_points_jumpman = 0;
						}
						else
						{
							bonus_points_total += BONUS_POINT_DECREMENT_VALUE;
							game_score += BONUS_POINT_DECREMENT_VALUE;
							bonus_points_jumpman -= BONUS_POINT_DECREMENT_VALUE;
						}
					}

					else // if (bonus_points_jumpman <= 0)
					{
						bp_complete = true;
						bp_complete_count = game_stage_count + 50;
					}
				}
				else if (game_stage_count > bp_complete_count)
				{
					InsertHighScores(game_score, bonus_points_total);

					SetGameStage(STAGE_high_scores_1);
				}

				GrayUserDrawStr(x_pos,92, buffer, COLOR_White, 0x00);
				break;
			}

			case STAGE_high_scores_1:
			{
				if (DrawHighScores() == false)
					SetGameStage(STAGE_high_scores_2);

				break;
			}

			case STAGE_high_scores_2:
			{
				DrawHighScores();

				if (game_stage_count > 100)
				{
					reset_game = true;
				}
				else if (game_stage_count > 25)
				{
					if (_keytest (RR_2ND))
						reset_game = true;
				}

				break;
			}

			case STAGE_pause_game:
				break;

			default:
				break;

		}

		if (exit_game || reset_game)
		{
			done = true;
			SaveHighScores();
		}

		frame_count++;
		game_stage_count++;

		Sleep(sleep_time);
		SwapPlaneBuffers();

	}
	
}

void UpdateJumpman(void)
{
	ANIM_Type* 	animation = (ANIM_Type*)&action_anim_sequence[jumpman.action];
	int2 sprite_index_adjust = 0;
	int1 x_direction_adjust  = 1;
	int1 y_direction_adjust  = 1;

	if (jumpman.direction == DIR_Left)
	{
		x_direction_adjust = -1;

		if ((jumpman.action == ACTION_Walk) || (jumpman.action == ACTION_Jump))
			sprite_index_adjust = 1;
	}
	else if (jumpman.direction == DIR_Down)
	{
		y_direction_adjust = -1;
	}

	jumpman.x += (int2)(animation->x_offset[jumpman.anim_step_cur] * x_direction_adjust);
	jumpman.y += (int2)(animation->y_offset[jumpman.anim_step_cur] * y_direction_adjust);

	jumpman.sprite_index = animation->sprite_index[jumpman.anim_step_cur] + sprite_index_adjust;
	jumpman.anim_step_cur++;

	DrawJumpman(jumpman.sprite_index, jumpman.x, jumpman.y);
}

void DrawJumpman(int2 sprite_index, int2 x, int2 y)
{
	uint1* pSprite = (uint1*)j3 + (sprite_index * (1*8*3));

	jumpman.x_orig = x;
	jumpman.y_orig = y;

// <+debrouxl> Ranman: for the time being, if you want a 8xh sprite, you can emulate ClipSprite8Get_R with 1) memset(sprite, 0, h); and then 2) passing sprite-y, where y < 0, to ClipSprite8Get_R :)
	ClipSprite8Get_R(x, y, 12, plane_active_0, jumpman_bkgd_0);
	ClipSprite8Get_R(x, y, 12, plane_active_1, jumpman_bkgd_1);

	if (y <= -12)
		y = -12;

	if (y < 0)
	{
		memset(jumpman_bkgd_0, 0x00, -y); // was 0xff when using visual plane
		memset(jumpman_bkgd_1, 0x00, -y); // was 0xff when using visual plane
	}

  if (jumpman.visible)
		GrayClipSprite8_SMASK_R(x, y, 8, pSprite + 0, pSprite + 8, pSprite + 16, PLANE_CurrentPlane0, PLANE_CurrentPlane1);

	if (jumpman.special_ability == SPECIAL_ABILITY_Illuminate)
	{
		Illuminate();
	}

	if (jumpman.special_ability == SPECIAL_ABILITY_Hotfoot)
	{
		Hotfoot();
	}

	if (pickup_platform)
	{
		if (jumpman.action <= ACTION_Fall)
		{
			pickup_platform->x = x;
			pickup_platform->y = y+2;

			if ((pickup_platform->x == dropoff_pos.x) && (abs(pickup_platform->y - dropoff_pos.y) <= 2))
			{
				pickup_platform->flags = FLAG_VA;
				pickup_platform->y = dropoff_pos.y;
				DrawPlatformPiece(pickup_platform->game_piece, pickup_platform->flags, pickup_platform->x, pickup_platform->y);
				dropoff_pos.y -= 4;
				pickup_platform = 0;
			}
			else
			{
				//DrawPlatformPiece(pickup_platform->game_piece, pickup_platform->flags, pickup_platform->x, pickup_platform->y);
				pSprite = (uint1*)j7 + (pickup_platform->game_piece * (1*4*3));
				GrayClipSprite8_SMASK_R(pickup_platform->x, pickup_platform->y, 4, pSprite + 0, pSprite + 4, pSprite + 8, PLANE_CurrentPlane0, PLANE_CurrentPlane1);

			}
		}
		else
		{
			pickup_platform->flags = FLAG_NONE;
			pickup_platform = 0;
		}
	}

}

void UpdateJumpmanMoveHistory(void)
{
	//if ((jumpman.action >= ACTION_Walk) && (jumpman.action <= ACTION_Rope))
	if (jumpman.action <= ACTION_Rope)
	{
		MOVE_History_Type* move_history_entry = &move_history[move_history_index & MOVE_HISTORY_MASK];

		move_history_entry->x = jumpman.x;
		move_history_entry->y = jumpman.y;
		move_history_entry->sprite_index = jumpman.sprite_index;
		move_history_index++;
	}
	else if (jumpman.action > ACTION_Rope)
	{
		move_history_index = 0;
	}
}

void CheckKeyPress(void)
{
	ACTION_Type jumpman_action_cur;
	ACTION_Type jumpman_action_new;
	bool allow_walking = false;
	bool left_or_right_direction = false;
	bool jump_in_progress = false;
	bool jump_is_interruptible = false;
	static int2 dead_count = 0;
//	static int2 count = 0;
//	char buffer[16];

	DIR_Type direction = DIR_None;
	bool action_key = false;
	bool jump_key = false;

	if (titlescreen)
	{
		GetScriptedMove(&direction, &action_key);
	}
	else
	{
		if (jumpman.freeze_count)
		{
			if (--jumpman.freeze_count <= 0)
			{
				jumpman.freeze_count = 0;
			}
		}
		else
		{
			GetPlayerKeyPress(&direction, &action_key);
		}
	}

	if (direction || action_key)
	{
		ResetAutoQuit();
	}

	if (jumpman.special_ability == SPECIAL_ABILITY_Gunfire)
	{
		if (jumpman.bullet_fire_count == 0)
		{
			jumpman.bullet_fire_count = 8;
			
			if (action_key)
				FireBullet(0, GAME_PIECE_Jumpman_Gunfire, direction);
		}
		else
		{
			jumpman.bullet_fire_count--;
		}
	}
	else if (jumpman.special_ability == SPECIAL_ABILITY_Spear)
	{
		if ((action_key) && (direction != DIR_Up) && (direction != DIR_Down))
			FireBullet(0, GAME_PIECE_Spear, direction);
		else
			jump_key = action_key;
	}
	else
	{
		jump_key = action_key;
	}

	CheckPlatformInteractions(direction);


	jumpman_action_cur = jumpman.action;
	jumpman_action_new = jumpman_action_cur;

	if ((jumpman_action_cur == ACTION_Jump) || (jumpman_action_cur == ACTION_Jump_Up))
	{
		jump_in_progress = true;

		direction = jumpman.direction;

		if (jumpman.anim_step_cur >= 5)
		{
			jump_is_interruptible = true;
		}
	}

	if ((direction == DIR_Left) || (direction == DIR_Right))
	{
		left_or_right_direction = true;
	}

	if (CheckJumpmanGirderRow(2, 11))
	{
		allow_walking = true;
	}


	if (jumpman_action_cur == ACTION_Dead)
	{
		direction = DIR_None;

		if (dead_count++ >= 50)
		{
			dead_count = 0;
			jumpman_dead = true;
		}
	}

	else if (jumpman_action_cur == ACTION_Fall)
	{
		//GrayUserDrawStr(74,1, "Fall", COLOR_White, 0);
		direction = DIR_None;

		if ((CheckJumpmanGirderRow(8,9)))// && (jumpman.anim_step_cur > 0))
		{

			jumpman_action_new = ACTION_Bounce;

			if (jumpman.x < 4)
				direction = DIR_Right;
			else if (jumpman.x > 155)
				direction = DIR_Left;
			else
				direction = DIR_Left + RandomNumber(2);
		}
		else if (jumpman.y >= 84)
		{
			jumpman_action_new = ACTION_Dead;
			dead_count = 0;
		}
	}

	else if (jumpman_action_cur == ACTION_Bounce)
	{
		//GrayUserDrawStr(74,1, "Bounce", COLOR_White, 0);
		direction = jumpman.direction;

		if (jumpman.anim_step_cur >= jumpman.anim_step_max)
		{
			//GrayUserDrawStr(74,10, "Bounce", COLOR_White, 0);
			jumpman_action_new = ACTION_Fall;
			direction = DIR_None;

			//jumpman.anim_step_cur = 0;
			//jumpman.y+=2;
		}

		if (CheckJumpmanGirderRow(8,9))
		{
			if (jumpman.y >= 80)
			{
				jumpman_action_new = ACTION_Dead;
				direction = DIR_None;
				dead_count = 0;
			}
		}
	}

	else if (jumpman.y >= 85)
	{
		jumpman_action_new = ACTION_Dead;
		direction = DIR_None;
		jumpman.y = 84;
		
	}

	else if (jumpman.y <= -16)
	{
		jumpman_action_new = ACTION_Fall;
		direction = DIR_None;
	}

	//else if (jumpman.x <= -6) // does not work with wrap mode
	else if (jumpman.x <= -4)
	{
		if (current_level.wrap_mode)
		{
			jumpman.x = 155;
		}
		else
		{
			jumpman_action_new = ACTION_Fall;
			direction = DIR_None;
		}
	}

	//else if (jumpman.x >= 158) // does not work with wrap mode
	else if (jumpman.x > 156)
	{
		if (current_level.wrap_mode)
		{
			jumpman.x = -3;
		}
		else
		{
			jumpman_action_new = ACTION_Fall;
			direction = DIR_None;
		}
	}

	else if ((jump_in_progress) && (!jump_is_interruptible))
	{
		//  we are at the beginning of a jump
		//  so do not allow any interactions
	}

	else if (CheckJumpmanLadder())
	{
		if ((jumpman_action_cur != ACTION_Ladder) &&
				(jumpman_action_cur != ACTION_Ladder_Still) &&
				((!left_or_right_direction) ||
				((left_or_right_direction) && (jump_in_progress) && (jump_is_interruptible))))
		{
			jumpman_action_new = ACTION_Ladder_Still;
		}

		if (jump_in_progress == false)
		{
			jumpman_action_new = ACTION_Ladder_Still;

			if (direction == DIR_Up)
			{
				//GrayUserDrawStr(74,1, "Ladder Up", COLOR_White, 0);
				if (jump_key)
				{
					jumpman_action_new = ACTION_Jump_Up;
				}
				else if (CheckJumpmanLadderRow(1, 5))
				{
					jumpman_action_new = ACTION_Ladder;
				}
			}
			else if (direction == DIR_Down)
			{
				//GrayUserDrawStr(74,1, "Ladder Down", COLOR_White, 0);
				if ((CheckJumpmanGirderRow(7,8)) && (!CheckJumpmanLadderRow(8, 8)))
				//if (!CheckJumpmanLadderRow(8, 8))
				{
					jumpman_action_new = ACTION_Ladder_Still;
				}
				else
				{
						jumpman_action_new = ACTION_Ladder;
				}
			}
			else if (left_or_right_direction)
			{
				//GrayUserDrawStr(74,1, "Ladder L/R", COLOR_White, 0);
				if (jump_key)
				{
					jumpman_action_new = ACTION_Jump;
				}
				else if ((allow_walking))// || CheckJumpmanGirderRow(10, 10))
				{
					jumpman_action_new = ACTION_Walk;
				}
			}
			else // if (direction == DIR_None)
			{
				// do nothing
				if (allow_walking)
				{
					if (current_level.wind_speed != 0)
						jumpman_action_new = ACTION_None;
				}
			}
		}
		else
		{
			if (allow_walking)
			{
				if (current_level.wind_speed == 0)
					jumpman_action_new = ACTION_Ladder_Still;
			}
		}
	}

	else if (CheckJumpmanRope())
	{
		jumpman_action_new = ACTION_Rope;

		//GrayUserDrawStr(74,1, "Rope", COLOR_White, 0);

		if (left_or_right_direction)
		{
			if ((jump_key) && (!jump_in_progress))
			{
				jumpman_action_new = ACTION_Jump;
			}
			else if (allow_walking)
			{
				//GrayUserDrawStr(74 + 4*5,1, "Skip", COLOR_White, 0);
				jumpman_action_new = ACTION_Walk;
				skip_rope_check = true;
			}
		}
		else
		{
			direction = DIR_None;
		}
	}

//	else if ((CheckJumpmanChain()) ||
//					 ((CheckJumpmanGirderRow(2,7)) && (direction == DIR_None)))
	else if (CheckJumpmanChain())
	{
		jumpman_action_new = ACTION_Chain;

		//GrayUserDrawStr(74,1, "Chain", COLOR_White, 0);

		if (left_or_right_direction)
		{
			if ((jump_key) && (!jump_in_progress))
			{
				jumpman_action_new = ACTION_Jump;
			}
			else if (allow_walking)
			{
				//GrayUserDrawStr(74 + 4*6,1, "Skip", COLOR_White, 0);
				jumpman_action_new = ACTION_Walk;
				skip_chain_check = true;
			}
		}
		else
		{
			direction = DIR_None;
		}
	}

	else if (CheckJumpmanGirderRow(2,6))
	{
		//GrayUserDrawStr(74,1, "Girder 2,6", COLOR_White, 0);

		if ((allow_walking) && (left_or_right_direction))
		{
			jumpman_action_new = ACTION_Walk;
			jumpman.y--;
		}
		else
		{
			jumpman_action_new = ACTION_Chain;
			direction = DIR_None;
		}
	}

	else if (CheckJumpmanGirderRow(7,7))
	{
		//GrayUserDrawStr(74,1, "Girder 7,7", COLOR_White, 0);

		if (jumpman_action_cur != ACTION_Walk)
		{
			jumpman_action_new = ACTION_None;
			direction = DIR_None;
		}

		jumpman.y--;
	}

//	else if ((!CheckJumpmanGirderRow(8,8)) && (CheckJumpmanGirderRow(9,9)))
//	{
//		jumpman.y++;
//	}

	else if ((jump_in_progress) &&
					 (jumpman.anim_step_cur >= 7))
	{
		if (CheckJumpmanGirderRow(8,8))
		{
			//GrayUserDrawStr(74,1, "Girder 8,9", COLOR_White, 0);

			jumpman_action_new = ACTION_None;
			direction = DIR_None;
		}
		else if (CheckJumpmanGirderRow(9,9))
		{
			//GrayUserDrawStr(74,1, "Girder 8,9", COLOR_White, 0);

			jumpman_action_new = ACTION_None;
			direction = DIR_None;
			jumpman.y++;
		}

		if (CheckJumpmanLadder())
			jumpman_action_new = ACTION_Ladder;
	}

	else
	{
		//GrayUserDrawStr(50,1, "Other", COLOR_White, 0);

		if (jump_in_progress == false)
		{
			//if (jumpman_action_cur != ACTION_Walk)
				jumpman_action_new = ACTION_None;
//jumpman_action_new = jumpman_action_cur;
			if (direction == DIR_Up)
			{
				if (jump_key)
				{
					jumpman_action_new = ACTION_Jump_Up;
				}
			}
			else if (left_or_right_direction)
			{
				//GrayUserDrawStr(50 + 4*6,1, "l/r", COLOR_White, 0);
				if (jump_key)
				{
					//GrayUserDrawStr(50 + 4*10,1, "jump", COLOR_White, 0);
					jumpman_action_new = ACTION_Jump;
				}
				else if ((allow_walking) || (CheckJumpmanGirderRow(9,9)))
				{
					//GrayUserDrawStr(50 + 4*10,1, "walk", COLOR_White, 0);
					jumpman_action_new = ACTION_Walk;
				}
			}

			if (!CheckJumpmanGirderRow(8,8))
			{

				//GrayUserDrawStr(50 + 6*4,1, "Girder 8,8", COLOR_White, 0);

				jumpman.y++;

				if (!CheckJumpmanGirderRow(9,9))
				{
					//GrayUserDrawStr(50 + 6*4 + 10*4,1, ":9,9", COLOR_White, 0);
					jumpman.y++;

					//der jumpman_action_new = ACTION_Fall;
					//der direction = DIR_None;
					
				  if (!CheckJumpmanGirderRow(10,10))
				  {
						jumpman.y++;
					
						jumpman_action_new = ACTION_Fall;
						direction = DIR_None;

				  	//if (!CheckJumpmanGirderRow(11,11))
				  	//{
						//	jumpman_action_new = ACTION_Fall;
						//	direction = DIR_None;
						//}
					}
				}
			}
		}
	}


	if (jumpman_action_new != jumpman_action_cur)
	{
		SetJumpmanAction(jumpman_action_new, direction);
	}
	else
	{
		jumpman.direction = direction;
	}

	if ((jumpman.action <= ACTION_Jump) ||
			(jumpman.action == ACTION_Fall) ||
			(jumpman.action == ACTION_Bounce))
			jumpman.x += current_level.wind_speed;

	if (jumpman.anim_step_cur >= jumpman.anim_step_max)
	{
		jumpman.anim_step_cur = 0;

		if (jump_in_progress)
		{
			SetJumpmanAction(ACTION_None, DIR_None);
		}
	}

/*
	if (skip_rope_check)
		skip_rope_check--;

	if (skip_chain_check)
		skip_chain_check--;
*/
}

void SetJumpmanAction(ACTION_Type action, DIR_Type direction)
{
	jumpman.direction = direction;
	jumpman.action = action;
	jumpman.anim_step_cur = 0;
	jumpman.anim_step_max = action_anim_sequence[action].anim_step_max;

	if ((action == ACTION_Jump) || (action == ACTION_Jump_Up))
	{
	  jumpman.x_jump = jumpman.x;
	  jumpman.y_jump = jumpman.y;
	}
	
	if ((action == ACTION_Fall) || (action == ACTION_Dead))
	{
	  jumpman.visible = true;
	}
		
	if ((action == ACTION_Dead) && (current_level.toggle_bits & TOGGLE_BIT_Enabled))
	{
		ToggleVisibilityObjectType(GAME_PIECE_Girder_Hor, GAME_PIECE_Rope_Up, true);
		ToggleVisibilityObjectType(GAME_PIECE_Rope_Down, GAME_PIECE_Ladder, true);
		ToggleVisibilityObjectType(GAME_PIECE_Bomb, GAME_PIECE_Bomb, true);
		#if 0
		ToggleVisibilityObjectType(GAME_PIECE_Girder_Hor,  true);
		ToggleVisibilityObjectType(GAME_PIECE_Girder_Ver,  true);
		ToggleVisibilityObjectType(GAME_PIECE_Girder_Up,   true);
		ToggleVisibilityObjectType(GAME_PIECE_Girder_Down, true);
		ToggleVisibilityObjectType(GAME_PIECE_Rope_Down,   true);
		ToggleVisibilityObjectType(GAME_PIECE_Rope_Up,     true);
		ToggleVisibilityObjectType(GAME_PIECE_Ladder,      true);
		ToggleVisibilityObjectType(GAME_PIECE_Bomb,        true);
		#endif
	}
	//else	if (action == ACTION_Fall)
	//{
	//	//jumpman.y += 5;
	//}


}


