// C Source File
// Created 11/21/2010; 6:34:56 AM

//******************************************************************************
//
// Project:         Jumpman 89
//
// file name:       jm_moving_platforms.c
//
// author:          David Randall "Ranman" 
//                  derandall99@gmail.com
//
// notes::          Jumpman was originally created by Randy Glover for the
//                  Atari 400, Commodore 64, IBM PCjr, and Apple II in 1982
//
//******************************************************************************


#include <tigcclib.h>
#include "extgraph.h"

#include "BasicTypes.h"
#include "BasicUtils.h"
#include "GrayUtils.h"
#include "FontUtils.h"

#include "jm_moving_platforms.h"
#include "jm_platforms.h"
#include "jm_objects.h"
#include "jm_graphics.h"
#include "jm_jumpman.h"

#if 1
MOVING_PLATFORM_OBJECT_Type  moving_platform_list[MOVING_PLATFORM_COUNT_MAX];

OBJECT_Type* pickup_platform;

ucoord1 dropoff_pos;


int2 moving_platform_count;


void DrawMovingPlatform(MOVING_PLATFORM_OBJECT_Type* moving_platform)
{
	int2 i;
	int2 x = moving_platform->x;
	int2 y = moving_platform->y;
	int2 length = moving_platform->object->waypt_and_length & 0x07;

	for (i = 0; i < length; i++)
	{
		ModifyPlatformPiece(moving_platform->game_piece, moving_platform->object->flags, x, y);

		if (moving_platform->game_piece == GAME_PIECE_Girder_Hor)
		{
			x += 4;
		}
		else if (moving_platform->game_piece == GAME_PIECE_Girder_Up)
		{
			x += 4;
			y -= 1;
		}
		else if (moving_platform->game_piece == GAME_PIECE_Girder_Down)
		{
			x += 4;
			y += 1;
		}
		else
		{
			y += 4;
		}
	}
}



void EraseMovingPlatform(MOVING_PLATFORM_OBJECT_Type* moving_platform)
{
	int2 i;
	int2 x = moving_platform->x;
	int2 y = moving_platform->y;
	int2 length = moving_platform->object->waypt_and_length & 0x07;

	for (i = 0; i < length; i++)
	{
		ModifyPlatformPiece(moving_platform->game_piece, FLAG_NONE, x, y);

		if (moving_platform->game_piece == GAME_PIECE_Girder_Hor)
		{
			x += 4;
		}
		else if (moving_platform->game_piece == GAME_PIECE_Girder_Up)
		{
			x += 4;
			y -= 1;
		}
		else if (moving_platform->game_piece == GAME_PIECE_Girder_Down)
		{
			x += 4;
			y += 1;
		}
		else
		{
			y += 4;
		}
	}
}



void DrawMovingPlatforms(void)
{
	MOVING_PLATFORM_OBJECT_Type* moving_platform = &moving_platform_list[0];
	int2 i;

	for (i = 0; i < moving_platform_count; i++)
	{
		DrawMovingPlatform(moving_platform);

		moving_platform++;
	}
}



void DrawMovablePlatformPiece(uint1 game_piece, uint1 flags, int2 x, int2 y)
{
	uint1* pSprite = (uint1*)j7 + (game_piece * (1*4*3));

//void GraySprite8_SMASK_R(unsigned short x asm("%d0"), unsigned short y asm("%d1"), unsigned short h asm("%d2"), const unsigned char *sprt0, const unsigned char *sprt1, const unsigned char *mask, void *dest0 asm("%a0"), void *dest1 asm("%a1")) __attribute__((__stkparm__));

	if (flags & FLAG_VISIBLE)
		GraySprite8_SMASK_R(x, y, 4, pSprite, pSprite+4, pSprite+8, plane_visible_0, plane_visible_1);

	if (flags & FLAG_ACTIVE)
	{
		if (game_piece <= GAME_PIECE_Rope_Up)
			Sprite8_OR_R(x, y, 4, pSprite+4, plane_active_1);
		else
			Sprite8_OR_R(x, y, 4, pSprite, plane_active_0);
	}
}



void UpdateMovingPlatforms(void)
{
	MOVING_PLATFORM_OBJECT_Type* moving_platform = &moving_platform_list[0];
	int2 i;
	int2 x_diff;
	int2 y_diff;
	int2 waypt_x;
	int2 waypt_y;

	//char temp[32];
	//sprintf(temp, "mps = %d", moving_platform_count);
	//GrayUserDrawStr(74,1, temp, COLOR_White, 0);

	for (i = 0; i < moving_platform_count; i++)
	{
		if ((moving_platform->object->flags & FLAG_HOLD) != FLAG_HOLD)
		{
			EraseMovingPlatform(moving_platform);

			if (moving_platform->object->flags & FLAG_FALLING)
			{
				moving_platform->y += moving_platform->speed;
			}/*
			else if (moving_platform->object->flags & FLAG_PICKUP)
			{
				//moving_platform->x = jumpman.x;
				//moving_platform->y = jumpman.y;
				moving_platform->x = move_history[(move_history_index-1) & MOVE_HISTORY_MASK].x;
				moving_platform->y = move_history[(move_history_index-1) & MOVE_HISTORY_MASK].y;

				if (moving_platform->x < 0)
					moving_platform->x = 0;
				if (moving_platform->y < 0)
					moving_platform->y = 0;
			}*/
			//der else if ((moving_platform->object->flags & FLAG_MOVING) == FLAG_MOVING)
			else if (moving_platform->uses_waypoints)
			{
				if (moving_platform->y > 88)
					moving_platform->y = 0;
				
				waypt_x = (int2)moving_platform->waypoint_list[moving_platform->waypoint_index].x * 4;
				waypt_y = (int2)moving_platform->waypoint_list[moving_platform->waypoint_index].y;

				if (BOUNDS_COLLIDE2HW(moving_platform->x, moving_platform->y, waypt_x, waypt_y, 1, 3, 1, 3))
				{
					moving_platform->waypoint_index++;

					if (moving_platform->waypoint_index >= moving_platform->waypoint_total)
					{
						moving_platform->waypoint_index = 0;
					}
				}

				x_diff = moving_platform->x - waypt_x;
				y_diff = moving_platform->y - waypt_y;

				if (x_diff < 0)
					moving_platform->x += moving_platform->speed;
				else if (x_diff > 0)
					moving_platform->x -= moving_platform->speed;

				if (y_diff < 0)
					moving_platform->y += moving_platform->speed;
				else if (y_diff > 0)
					moving_platform->y -= moving_platform->speed;
			}
			else if (moving_platform->uses_boundaries)
			{
				// have we reached a boundary
				if ((moving_platform->x <= moving_platform->boundary_topleft.x) ||
						(moving_platform->x >= moving_platform->boundary_bottomright.x))
				{
					moving_platform->speed_x *= (-1);
				}
				
				if ((moving_platform->y <= moving_platform->boundary_topleft.y) ||
						(moving_platform->y >= moving_platform->boundary_bottomright.y))
				{
					moving_platform->speed_y *= (-1);
				}
				
				moving_platform->x += moving_platform->speed_x;
				moving_platform->y += moving_platform->speed_y;
			}
		}

		moving_platform++;
	}
}



void CheckMovingPlatforms(void)
{
	MOVING_PLATFORM_OBJECT_Type* moving_platform = &moving_platform_list[0];
	int2  i;
	uint1 test_value = 0xf0;
	uint1 active_bkgd;
//	DIR_Type direction = DIR_None;


	for (i = 0; i < moving_platform_count; i++)
	{
		if ((moving_platform->object->flags & FLAG_HOLD) != FLAG_HOLD)
		{
			if (moving_platform->object->flags & FLAG_FALLING)
			{
				//Sprite8Get_R(moving_platform->x, moving_platform->y+2, 1, plane_active_1, &test_value);
				Sprite8Get_R(moving_platform->x, moving_platform->y+3, 1, plane_active_1, &active_bkgd);

				if (test_value & active_bkgd)
				{
					// the falling platform has hit something
					moving_platform->object->flags |= FLAG_HOLD;
				}
				else if (moving_platform->y >= 88)
				{
					// the falling platform has hit the bottom
					moving_platform->object->flags |= FLAG_HOLD;
				}

				if (BOUNDS_COLLIDE2HW(moving_platform->x, moving_platform->y + 1, jumpman.x + 1, jumpman.y + 1, 3, 6, 4, 6))
				{
					// the falling platform has hit Jumpman
					if (jumpman.action < ACTION_Fall)
					{
						SetJumpmanAction(ACTION_Fall, DIR_Down);
					}
				}
			}
			else if (moving_platform->uses_boundaries)
			{
				// have we reached a boundary
				if (moving_platform->game_piece == GAME_PIECE_Ladder)
				{
					if ((jumpman.y >= moving_platform->y-6) && (jumpman.y <= moving_platform->y+moving_platform->length*4-2))
					{
						if ((jumpman.x >= moving_platform->x-2) && (jumpman.x <= moving_platform->x+2))
						{ 
							jumpman.x = moving_platform->x;
							
							if ((frame_count & 0x2) == 0x2)
								jumpman.y += moving_platform->speed_y;
						}
					}
				}
			}
#if 0			
			//der else if (moving_platform->object->flags & FLAG_MOVING)
			else if (moving_platform->uses_waypoints)
			{
				// have we reached a waypoint?
				// if so, calculate new direction
				int2 waypt_x = (int2)moving_platform->waypoint_list[moving_platform->waypoint_index].x * 4;
				int2 waypt_y = (int2)moving_platform->waypoint_list[moving_platform->waypoint_index].y;

				//direction = moving_platform->direction;

				//if (direction == DIR_Up)


				if (BOUNDS_COLLIDE2HW(moving_platform->x, moving_platform->y, waypt_x-1, waypt_y-1, 1, 3, 1, 3))
				{
					moving_platform->waypoint_index++;
					//GrayUserDrawStr(74,10, "New wp", COLOR_White, 0);

					if (moving_platform->waypoint_index >= moving_platform->waypoint_total)
					{
						moving_platform->waypoint_index = 0;
						//GrayUserDrawStr(74,20, "End wp", COLOR_White, 0);
					}
				}
			}
#endif
		}

		moving_platform++;
	}
}



void PickUpPlatform(OBJECT_Type* object)
{
	if (pickup_platform)
	{
		pickup_platform->flags = FLAG_NONE;
		pickup_platform = 0;
	}
	
	if (object)
	{
		pickup_platform = object;
		pickup_platform->flags = FLAG_VP;
	}
}

// Simple
//{ 52, GAME_PIECE_Ladder,      FLAG_VAMW,  17, 16,  (10 << 3) | 2},

// Extended 	
//{ 70, GAME_PIECE_Ladder,       FLAG_VAMX,   71, 72,  6}, // the Sreddal
//{ 71, GAME_PIECE_Moving_Data,  FLAG_D_DIAG, 19,  0,  0}, // Sreddal movement data
//{ 72, GAME_PIECE_Boundary_Data,         12, 16, 28, 64}, // Sreddal boundaries


void InitMovingPlatforms(void)
{
	MOVING_PLATFORM_OBJECT_Type* moving_platform = &moving_platform_list[0];
  GAME_PIECE_Type moving_platform_type = GAME_PIECE_Girder_Hor; //der GAME_PIECE_First_Platform;
	OBJECT_Type* object;
	OBJECT_Type* waypoint_object;     // for simple only
	OBJECT_Type* boundary_object;     // for extended only
	OBJECT_Type* moving_data_object;  // for extended only
	int2 i;
	bool done = false;

	pickup_platform = 0;
	dropoff_pos.x = 19*4;
	dropoff_pos.y = 36;
	moving_platform_count = 0;
	memset((void*)(moving_platform_list), 0, sizeof(MOVING_PLATFORM_OBJECT_Type) * MOVING_PLATFORM_COUNT_MAX);

	while ((!done) && (moving_platform_count < MOVING_PLATFORM_COUNT_MAX))
	{
		object = FindObjectEntryByType(moving_platform_type, false);

		if (object)
		{
			//if (((object->game_piece == GAME_PIECE_Girder_Hor) || 
			//		 (object->game_piece == GAME_PIECE_Ladder)) && 
			//		 (object->flags & (FLAG_MOVABLE)))
			if (object->flags & (FLAG_MOVABLE))
			{
				// retain a pointer to the object
				moving_platform->object = object;
				moving_platform->game_piece = object->game_piece;
				
				if (object->flags & (FLAG_EXTENDED))
				{
					moving_platform->length = object->length;
					
					moving_data_object = FindObjectEntryById(object->moving_data_id, false);
					
					if (moving_data_object)
					{
						moving_platform->direction_flags = moving_data_object->dir_flags;
						moving_platform->x = moving_data_object->x * 4;
						moving_platform->y = moving_data_object->y;
						moving_platform->speed_x = moving_data_object->xy_speed >> 4;
						moving_platform->speed_y = moving_data_object->xy_speed & 0x0f;
					}
					else
					{
						moving_platform->object->flags |= FLAG_HOLD;
					}
					
					boundary_object = FindObjectEntryById(object->boundary_id, false);
					
					if (boundary_object)
					{
						moving_platform->boundary_topleft.x = boundary_object->topleft.x * 4;
						moving_platform->boundary_topleft.y = boundary_object->topleft.y;
						
						if (moving_platform_type == GAME_PIECE_Girder_Hor)
						{
							moving_platform->boundary_bottomright.x = (boundary_object->bottomright.x - moving_platform->length) * 4;
							moving_platform->boundary_bottomright.y = boundary_object->bottomright.y - 4;
						}
						else if (moving_platform_type == GAME_PIECE_Ladder)
						{
							moving_platform->boundary_bottomright.x = (boundary_object->bottomright.x - 2) * 4;
							moving_platform->boundary_bottomright.y = boundary_object->bottomright.y - (moving_platform->length * 4);
						}
						
						moving_platform->uses_boundaries = true;
					}
					else
					{
						moving_platform->object->flags |= FLAG_HOLD;
					}
				}
				else
				{
					moving_platform->x = object->x * 4;
					moving_platform->y = object->y;
					moving_platform->speed = 2;
					moving_platform->waypoint_index = 0;
					moving_platform->waypoint_total = 0;
					moving_platform->length = object->waypt_and_length & 0x7;;

					if (object->flags & FLAG_WAYPOINT)
					{
						waypoint_object = FindObjectEntryById(object->waypt_and_length >> 3, false);
						
						if (waypoint_object)
						{
							moving_platform->speed = waypoint_object->speed;
							moving_platform->waypoint_total = waypoint_object->count;
	
							for (i = 0; i < waypoint_object->count; i+= 2)
							{
								waypoint_object = FindObjectEntryById(waypoint_object->id+1, false);
								memcpy(&moving_platform->waypoint_list[i], &waypoint_object->pos[0], 4); // each waypoint data level entry contains 2 position coords
							}
							
							moving_platform->uses_waypoints = true;
						}

					}
					else 
					{
						moving_platform->object->flags |= FLAG_HOLD;
					}
				}

				moving_platform_count++;
				moving_platform++;
			}
		}
		else
		{
			if (moving_platform_type == GAME_PIECE_Ladder)
			 	done = true;
			else
				moving_platform_type = GAME_PIECE_Ladder;

			//der moving_platform_type++;
			
			//der if (moving_platform_type > GAME_PIECE_Last_Platform)
			//der 	done = true;
		}
	}
}

#endif
