// Header File
// Created 11/21/2010; 6:58:28 AM

//******************************************************************************
//
// Project:         Jumpman 89
//
// file name:       jm_moving_platforms.h
//
// author:          David Randall "Ranman" 
//                  derandall99@gmail.com
//
// notes::          Jumpman was originally created by Randy Glover for the
//                  Atari 400, Commodore 64, IBM PCjr, and Apple II in 1982
//
//******************************************************************************


#ifndef _jm_moving_platforms_h
#define _jm_moving_platforms_h

#include "BasicTypes.h"

#include "jm_enemies.h"

#define MOVING_PLATFORM_STOP_SPEED 0
#define MOVING_PLATFORM_NORM_SPEED 2
#define MOVING_PLATFORM_FALL_SPEED 2
#define MOVING_PLATFORM_COUNT_MAX  16


typedef struct
{
	uint2 game_piece;         // GAME_PIECE_Type
	OBJECT_Type* object;      // pointer to the object
	int2  x;                  // current lcd x coord
	int2  y;                  // current lcd y coord	
	
	bool is_extended;
	bool uses_boundaries;
	bool uses_waypoints;
	
	uint1 length;             // max of 7
	uint1 speed;              // speed of enemy;
	int1 speed_x;              // speed of enemy;
	int1 speed_y;              // speed of enemy;
	uint1 direction_flags;
	
  uint1 waypoint_index;     // current index into the waypoint list
  uint1 waypoint_total; //
	ucoord1 waypoint_list[WAYPOINT_COUNT_MAX];
		
	ucoord1 boundary_topleft;
	ucoord1 boundary_bottomright;

} MOVING_PLATFORM_OBJECT_Type;



extern int2 moving_platform_count;
extern MOVING_PLATFORM_OBJECT_Type  moving_platform_list[MOVING_PLATFORM_COUNT_MAX];

extern ucoord1 dropoff_pos;
extern OBJECT_Type  pickup_platform_list[MOVING_PLATFORM_COUNT_MAX];
extern OBJECT_Type* pickup_platform;


void DrawMovingPlatforms(void);
void UpdateMovingPlatforms(void);
void CheckMovingPlatforms(void);
void InitMovingPlatforms(void);

void DrawMovablePlatformPiece(uint1 game_piece, uint1 flags, int2 x, int2 y);
void PickUpPlatform(OBJECT_Type* object);



#endif
