// Header File
// Created 11/21/2010; 1:06:18 PM

//******************************************************************************
//
// Project:         Jumpman 89
//
// file name:       jm_objects.h
//
// author:          David Randall "Ranman" 
//                  derandall99@gmail.com
//
// notes::          Jumpman was originally created by Randy Glover for the
//                  Atari 400, Commodore 64, IBM PCjr, and Apple II in 1982
//
//******************************************************************************


#ifndef _jm_objects_h
#define _jm_objects_h

#include "BasicTypes.h"


typedef enum
{
	GAME_PIECE_None              = 0,

	// Platform IDs
	GAME_PIECE_Girder_Hor        = 1,
	GAME_PIECE_Girder_Ver,
	GAME_PIECE_Girder_Up,
	GAME_PIECE_Girder_Down,
	GAME_PIECE_Rope_Up,
	GAME_PIECE_Rope_Down, // always on top
	GAME_PIECE_Ladder, // always on top

	// Collectable IDs
	GAME_PIECE_Bomb              = 16,
	GAME_PIECE_Runaway_Bomb,
	GAME_PIECE_Jewel,
	GAME_PIECE_Trigger,

	// Bullet IDs
	GAME_PIECE_Bullet            = 32, // this is the first bullet object
	GAME_PIECE_Jumping_Block,
	GAME_PIECE_Hailstone,
	GAME_PIECE_Zig_Zag_Bullet,
	GAME_PIECE_Enemy_Gunfire,
	GAME_PIECE_Jumpman_Gunfire,
	GAME_PIECE_Spear,

	// Enemy IDs
	GAME_PIECE_Falling_Bomb      = 48, // this is the first enemy object
	GAME_PIECE_Ufo,
	GAME_PIECE_Bat,
	GAME_PIECE_Fire,
	GAME_PIECE_Bird,
	GAME_PIECE_Follower,
	GAME_PIECE_Robot_1,
	GAME_PIECE_Robot_2,
	GAME_PIECE_Robot_3,
	GAME_PIECE_Hatchling,
	GAME_PIECE_Wall,
	GAME_PIECE_Door,
	GAME_PIECE_Transporter, 
	GAME_PIECE_Dragon,
	GAME_PIECE_Gunslinger, // this is the last enemy object
	GAME_PIECE_Barrel,
	GAME_PIECE_Freezer, // same sprite as hatchling
	GAME_PIECE_Falcon,
	GAME_PIECE_Transporter2,
  
	// Misc IDs
	GAME_PIECE_Jumpman           = 80,
	GAME_PIECE_Script,
	GAME_PIECE_Waypoint_Root,
	GAME_PIECE_Waypoint_List_Pos,
	GAME_PIECE_Waypoint_List_Vec,
	GAME_PIECE_Timer,
	GAME_PIECE_Boundary_Data,
	GAME_PIECE_Moving_Data,


	GAME_PIECE_First_Platform    = GAME_PIECE_Girder_Hor,
	GAME_PIECE_Last_Platform     = GAME_PIECE_Ladder,

	GAME_PIECE_First_Collectable = GAME_PIECE_Bomb,
	GAME_PIECE_Last_Collectable  = GAME_PIECE_Trigger,

	GAME_PIECE_First_Bullet      = GAME_PIECE_Bullet,
	GAME_PIECE_Last_Bullet       = GAME_PIECE_Spear,

	GAME_PIECE_First_Enemy       = GAME_PIECE_Falling_Bomb,
	GAME_PIECE_Last_Enemy        = GAME_PIECE_Transporter2,

	//GAME_PIECE_Disable_Bit       = 0x80,
	
} GAME_PIECE_Type;

#if 0
typedef struct
{
	uint1 id;               // 1-255, 0 = End
	uint1 game_piece;       // GAME_PIECE_Type
	uint1 flags;            // visible,active,vertical,drop
	union
	{
		uint1 x;              // starting lcd x coord
		uint1 script_flags;   // visible,active,vertical,drop
	};
	union
	{
		uint1 y;              // starting lcd y coord
		uint1 game_piece_id;  // the ID of the game piece to act upon
	};
	union
	{
		uint1 length;         // 1 unit = 4 pixels
		uint1 script_id;      // the ID of the script to execute (if script flag is set)
		uint1 action;  // BULLET_ACTION_Type (if game_piece == GAME_PIECE_Bullet)
		uint1 special_ability;  // SPECIAL_ABILITY_Type (if game_piece == GAME_PIECE_Jumpman or GAME_PIECE_Enemy)
	};
} OBJECT_Type;
#else
typedef struct
{
	union
	{
		struct platform
		{
			uint1 id;               // 1-255, 0 = End
			uint1 game_piece;       // GAME_PIECE_Type
			uint1 flags;            // visible,active,vertical,drop
			uint1 x;                // starting lcd x coord
			uint1 y;                // starting lcd y coord
			uint1 length;           // 1 length unit = 4 pixels
		};
		struct moving_platform // simple moving object
		{
			uint1 id;               // 1-255, 0 = End
			uint1 game_piece;       // GAME_PIECE_Type
			uint1 flags;            // visible,active,vertical,drop
			uint1 x;                // starting lcd x coord
			uint1 y;                // starting lcd y coord
			uint1 waypt_and_length; // top 5 bits = waypoint id; bottom 3 bits = length (1 length unit = 4 pixels)
		};
		struct moving_platform_ext // extended moving object
		{
			uint1 id;               // 1-255, 0 = End
			uint1 game_piece;       // GAME_PIECE_Type
			uint1 flags;            // visible,active,vertical,drop
			uint1 moving_data_id;                // starting lcd y coord
			uint1 boundary_id; // top 5 bits = waypoint id; bottom 3 bits = length (1 length unit = 4 pixels)
			uint1 length;                // starting lcd x coord
		};
		struct moving_data
		{
			uint1 id;               // 1-255, 0 = End
			uint1 game_piece;       // GAME_PIECE_Type
			uint1 dir_flags;      // the ID of the script to execute (if script flag is set)
			uint1 x;            // visible,active,vertical,drop
			uint1 y;            // visible,active,vertical,drop
			uint1 xy_speed;            // number of waypoints
		};
		struct boundary
		{
			uint1 id;               // 1-255, 0 = End
			uint1 game_piece;       // GAME_PIECE_Type
			ucoord1 topleft;        // visible,active,vertical,drop
			ucoord1 bottomright;    // visible,active,vertical,drop
		};
		struct collectible
		{
			uint1 id;               // 1-255, 0 = End
			uint1 game_piece;       // GAME_PIECE_Type
			uint1 flags;            // visible,active,vertical,drop
			uint1 x;                // starting lcd x coord
			uint1 y;                // starting lcd y coord
			uint1 script_id;        // the ID of the script to execute (if script flag is set)
		};
		struct script
		{
			uint1 id;               // 1-255, 0 = End
			uint1 game_piece;       // GAME_PIECE_Type
			uint1 flags;            // visible,active,vertical,drop
			union
			{
				uint1 object_flags;     // visible,active,vertical,drop
				uint1 command;          // visible,active,vertical,drop
			};
			uint1 game_piece_id;    // the ID of the game piece to act upon
			uint1 script_id;        // the ID of the script to execute (if script flag is set)
		};
		struct waypoint_root
		{
			uint1 id;               // 1-255, 0 = End
			uint1 game_piece;       // GAME_PIECE_Type
			uint1 flags;            // visible,active,vertical,drop
			uint1 speed;            // visible,active,vertical,drop
			uint1 count;            // number of waypoints
			uint1 waypoint_id;      // the ID of the script to execute (if script flag is set)
		};
		struct waypoint_list_pos
		{
			uint1 id;               // 1-255, 0 = End
			uint1 game_piece;       // GAME_PIECE_Type
			ucoord1 pos[2];         // GAME_PIECE_Typeuvect1
		};
		struct waypoint_list_vec
		{
			uint1 id;               // 1-255, 0 = End
			uint1 game_piece;       // GAME_PIECE_Type
			uvect1 vec[2];          // GAME_PIECE_Type
		};
		struct jumpman
		{
			uint1 id;               // 1-255, 0 = End
			uint1 game_piece;       // GAME_PIECE_Type
			uint1 flags;            // visible,active,vertical,drop
			uint1 x;                // starting lcd x coord
			uint1 y;                // starting lcd y coord
			uint1 special_ability;  // SPECIAL_ABILITY_Type (if game_piece == GAME_PIECE_Jumpman or GAME_PIECE_Enemy)
		};
		struct enemy
		{
			uint1 id;               // 1-255, 0 = End
			uint1 game_piece;       // GAME_PIECE_Type
			uint1 flags;            // visible,active,vertical,drop
			uint1 x;                // starting lcd x coord
			uint1 y;                // starting lcd y coord
			uint1 special_ability;  // SPECIAL_ABILITY_Type (if 'W' flag bit == 1 then this is waypoint ID)
		};
		struct bullet
		{
			uint1 id;               // 1-255, 0 = End
			uint1 game_piece;       // GAME_PIECE_Type
			uint1 flags;            // visible,active,vertical,drop
			uint1 x;                // starting lcd x coord
			uint1 y;                // starting lcd y coord
			uint1 action;           // BULLET_ACTION_Type (if game_piece == GAME_PIECE_Bullet)
		};
		struct wall
		{
			uint1 id;               // 1-255, 0 = End
			uint1 game_piece;       // GAME_PIECE_Type
			uint1 flags;            // visible,active,vertical,drop
			uint1 x;                // starting lcd x coord
			uint1 y;                // starting lcd y coord
			uint1 height_and_dir;   // height = 5 bits, dir = 3 bits
		};
		struct door
		{
			uint1 id;               // 1-255, 0 = End
			uint1 game_piece;       // GAME_PIECE_Type
			uint1 flags;            // visible,active,vertical,drop
			uint1 x;                // starting lcd x coord
			uint1 y;                // starting lcd y coord
			uint1 height_and_dir;   // height = 5 bits, dir = 3 bits
		};
		struct data_root
		{
			uint1 id;               // 1-255, 0 = End
			uint1 game_piece;       // GAME_PIECE_Type
			uint1 flags;            // visible,active,vertical,drop
			uint1 speed;            // visible,active,vertical,drop
			uint1 count;            // number of waypoints
			uint1 data_id;      // the ID of the script to execute (if script flag is set)
		};
		struct data
		{
			uint1 id;               // 1-255, 0 = End
			uint1 game_piece;       // GAME_PIECE_Type
			uint1 data[4];          // visible,active,vertical,drop
		};
		struct timer
		{
			uint1 id;               // 1-255, 0 = End
			uint1 game_piece;       // GAME_PIECE_Type
			uint1 flags;            // visible,active,vertical,drop
			uint1 timer_value_u;    // the ID of the game piece to act upon
			uint1 timer_value_l;    // the ID of the game piece to act upon
			uint1 timer_script_id;  // the ID of the script to execute (if script flag is set)
		};		
	};
} OBJECT_Type;
#endif


OBJECT_Type* FindObjectEntryById(uint1 id, bool start_over);
OBJECT_Type* FindObjectEntryByType(GAME_PIECE_Type type, bool start_over);
OBJECT_Type* GetNextObjectEntry(void);

void ResetIndexByType(void);


#endif