// C Source File
// Created 3/5/2005; 10:10:40 AM

//******************************************************************************
//
// file name:       FileUtils.c
//
// author:          David Randall "Ranman" 
//                  derandall99@gmail.com
//
//******************************************************************************


#include <tigcclib.h>

#include "BasicTypes.h"
#include "FileUtils.h"


HSym* GetHSym(char* fileName)
{
	static HSym hSym;
	HSym* pHSym = 0;

	// the requested folder was found
	hSym = SymFind (SYMSTR (fileName));

	if (hSym.folder)
	{
		// we found the symbol
		pHSym = &hSym;
	}

	return pHSym;
}

SYM_ENTRY* GetSymEntry(char* fileName)
{
	SYM_ENTRY* pSymEntry = 0;
	HSym* pHSym = GetHSym(fileName);

	if (pHSym)
	{
		pSymEntry = DerefSym (*pHSym);
	}

	return pSymEntry;
}

bool OpenFolder(char* folderName)
{
	return FolderCur (SYMSTR (folderName), TRUE);
}

void* OpenFile(char* fileName)
{
	void* fileAddress = 0;
	SYM_ENTRY* pSymEntry = GetSymEntry(fileName);

	if (pSymEntry)
	{
		HeapLock(pSymEntry->handle);

		fileAddress = HeapDeref (pSymEntry->handle) + 2;
	}

	return fileAddress;
}

bool CloseFile(char* fileName)
{
	SYM_ENTRY* pSymEntry = GetSymEntry(fileName);

	if (pSymEntry)
	{
		HeapUnlock(pSymEntry->handle);

		return 1;
	}

	return 0;
}

bool ArchiveFile(char* fileName)
{
	HSym* pHSym = GetHSym(fileName);

	if (pHSym)
	{
		return EM_moveSymToExtMem(NULL, *pHSym);
	}

	return 0;
}

bool UnarchiveFile(char* fileName)
{
	HSym* pHSym = GetHSym(fileName);

	if (pHSym)
	{
		return EM_moveSymFromExtMem(NULL, *pHSym);
	}

	return 0;
}

void* NewFile(char* fileName, void* pSource, uint2 size)
{
	// the save file does not exist, so we must create it
	HANDLE handle;

	// NOTE: HeapAlloc must be executed before DerefSym due to possible garbage collection.
	handle = HeapAllocHigh (2 + size);  // add 8 bytes to account for tag, type, and NULLs

	if (handle)
	{
		// allocation was successful, now add a new symbol to the VAT
		HSym hSym;

		//HeapCompress();
		//HeapMoveHigh(handle);
		HeapLock(handle);

		hSym = SymAdd (SYMSTR (fileName));

		if (hSym.folder)
		{
			MULTI_EXPR *pFileVar;

			SYM_ENTRY* pSymEntry = DerefSym(hSym);

			pSymEntry->handle = handle;

			// the symbol was created successfully
			pFileVar = HeapDeref (handle);

			// copy the contents to the file
			memcpy(pFileVar->Expr, pSource, size);

			// add the file size to the file header
			//pFileVar->Size = size - 1;
			pFileVar->Size = size;

			return pFileVar->Expr;
			//return OpenFile(fileName);
		}
		else
		{
			// the symbol was not created successfully, free the allocated memory
			HeapFree(handle);
		}
	}

	return 0;
}

char* FindFileType(char* fileType, bool findNext)
{
	if (fileType != NULL)
	{
		SYM_ENTRY* pSymEntry;
		static char file_name[10];
	
		memset (file_name, 0, 10);

		if (findNext == false)
			pSymEntry = SymFindFirst (NULL, FO_RECURSE);
		else
			pSymEntry = SymFindNext ();
	
	
		while (pSymEntry)
		{
  		bool found_type = false;
  		
			HeapLock(pSymEntry->handle);

			MULTI_EXPR *pFileVar = HeapDeref (pSymEntry->handle);
			char* pFileType = (char*)((uint4)pFileVar->Expr + pFileVar->Size - 5);
			
			if (strncmp(pFileType, fileType, 4) == 0)
				found_type = true;

			HeapUnlock(pSymEntry->handle);

			if (found_type)
			{
				return pSymEntry->name;

				//memcpy(file_name, pSymEntry->name, 8);
				//return file_name;
			}
			
			pSymEntry = SymFindNext ();
		}
	}

	return 0;
}


/*
int2 NewFile(
		char* folderName,
		char* fileName,
		char* fileExtension,
		uint2 fileSize)
{
	// the save file does not exist, so we must create it
	HANDLE handle;
	char fileExt[8] = {0,0,0,0,0,0,0,OTH_TAG};

	// NOTE: HeapAlloc must be executed before DerefSym due to possible garbage collection.
	handle = HeapAllocHigh (fileSize + 2 + sizeof(fileExt));  // add 8 bytes to account for tag, type, and NULLs

	if (handle)
	{
		// allocation was successful, now add a new symbol to the VAT
		HSym hSym;

		//HeapCompress();
		//HeapMoveHigh(handle);
		HeapLock(handle);

		hSym = SymAdd (SYMSTR (fileName));

		if (hSym.folder)
		{
			// the symbol was created successfully
			MULTI_EXPR *pFileVar = HeapDeref (handle);

			int2 extLength = strlen (fileExtension);

			if (extLength > 4)
				extLength = 4;

			// populate the full temporary file extension
			memcpy(fileExt + sizeof(fileExt) - (extLength + 2), fileExtension, extLength);

			// initialize the memory of the file
			memset(pFileVar->Expr, 0, fileSize);

			// add the file extension to the file footer
			memcpy(pFileVar->Expr + fileSize, fileExt, sizeof(fileExt));

			// add the file size to the file header
			pFileVar->Size = fileSize + sizeof(fileExt);

			pSymEntry = DerefSym(hSym);

			pSymEntry->handle = handle;

			HeapUnlock(handle);

			return 1;
		}
		else
		{
			// the symbol was not created successfully, free the allocated memory
			HeapFree(handle);
		}
	}

	return 0;
}



#include <stdio.h>
#include <tigcclib.h>

#define DEFAULT_FILE_ALLOC_SIZE 128
#define DEFAULT_FILE_BUFF_INC DEFAULT_FILE_ALLOC_SIZE

__ATTR_LIB_C__ FILE *fopen(const char *name, const char *mode)
{
  char str[50],*fpos,*epos,*base=0,*sptr=str,chmode=mode[0];
  short bmode=(mode[1]=='b'||(mode[1]&&mode[2]=='b')),flags=0,ferr;
  FILE *f;
  HSym hsym;
  HANDLE handle=H_NULL;

  *sptr=0; while((*++sptr=*name++));

  hsym=SymFind(sptr);

  if(hsym.folder)
    {
      SYM_ENTRY *sym_entry=DerefSym(hsym);
      if((sym_entry->flags.flags_n&(SF_LOCKED|SF_OPEN|SF_ARCHIVED|SF_BUSY))&&strpbrk(mode,"wa+"))
        return NULL;
      handle=sym_entry->handle;
    }
  else
    {
      return NULL;
    }

  if(!(f=malloc(sizeof(FILE)))) return NULL;

  if(chmode=='w')
    {
      flags = _F_WRIT;
      hsym = SymAdd(sptr);
      ferr = !(hsym.folder);
      if(!ferr)
        {
          handle=HeapAlloc(DEFAULT_FILE_ALLOC_SIZE);
          ferr=!(DerefSym(hsym)->handle=handle);
        }
      if(ferr)
        {
          SymDel(sptr);
          free(f);
          return NULL;
        }

      f->alloc=DEFAULT_FILE_ALLOC_SIZE;

      base=HLock(handle);

      if(bmode)
      {
      	poke_w(base,0);
      }
      else
      {
        poke_l(base,0x00050001);
        poke_l(base+4,0x2000E000);
      }
    }

  epos = base + peek_w(base) + (bmode?2:0);

  if(chmode == 'a')
  {
    flags = _F_WRIT;
    fpos = epos;
  }
  else
  {
    fpos=base+(bmode?2:5);
  }

  if(epos == fpos)
    flags |= _F_EOF;

  if(mode[1]=='+'||mode[2]=='+')
    flags|=_F_RDWR;

  if(bmode)
    flags|=_F_BIN;

  f->handle=handle;
  f->base=base;
  f->flags=flags;
  f->fpos=fpos;
  f->unget=0;
  f->buffincrement=DEFAULT_FILE_BUFF_INC;

  return f;
}


Christian Walther pointed to me that this is possible. Moreover, this is easy. All you need to do is to create a file of "other" type (i.e. which terminates with OTH_TAG). TIOS then expects that a real type name is stored just below OTH_TAG as a zero-started zero-terminated string of maximum 4 chars. Look the following code fragment as an example:
FILE *fp = fopen ("example", "wb");
// store anything you want in the file here
fputc (0, fp);
fputs ("HSC", fp);
fputc (0, fp);
fputc (OTH_TAG, fp);
fclose (fp);

After this, you will have a file named "example" with type "HSC": you will see it in the VAR-LINK dialog.


SYM_ENTRY* GetSymEntry(
		char* folderName,
		char* fileName)
{
	SYM_ENTRY* pSymEntry;
	char currentFolderName[8];

	// start the search, search every file in every folder
	pSymEntry = SymFindFirst (SYMSTR (fileName), FO_RECURSE);

	while (pSymEntry)
	{
		if (pSymEntry->flags.bits.folder)
		{
			// this is a folder, save the folder name
			strncpy(currentFolderName, pSymEntry->name, 8);
		}

		if (!strncmp(fileName, pSymEntry->name, 8))
		{
			// we found a matching file name
			if (*folderName)
			{
				// a folder name was specified, so ensure folder name matches
				if (!strncmp(folderName, currentFolderName, 8))
				{
					// the folder name matches too, now exit
					break;
				}
			}
			else
			{
				// disregard the folder name and exit
				break;
			}
		}

		// keep searching the VAT
		pSymEntry = SymFindNext();
	}

	return pSymEntry;
}


void* LoadFile(
		char* filePath,
		char* fileName)
{
	void* fileAddress = 0;
	SYM_ENTRY* pSymEntry = GetSymEntry(filePath, fileName);

	if (pSymEntry)
	{
		if (!HeapGetLock(pSymEntry->handle))
			HeapLock(pSymEntry->handle);

		fileAddress = HeapDeref (pSymEntry->handle) + 2;
	}

	return fileAddress;
}


bool SaveFile(
		char* filePath,
		char* fileName,
		char* fileExtension,
		uint4 bufferAddress,
		uint2 bufferSize)
{
	SYM_ENTRY* pSymEntry;

	pSymEntry = GetSymEntry(filePath, fileName);

	if (!pSymEntry)
	{
		// the save file does not exist
		// HeapAlloc must be executed before DerefSym
		// because of possible garbage collection.
		HANDLE handle = HeapAlloc (sizeof(GameData));

		// set the current folder
		FolderCur (SYMSTR (filePath), FALSE);

		// create the new file
		pSymEntry = DerefSym (SymAdd (SYMSTR (fileName)));
	}

	if (pSymEntry)
	{
		// the save file is ready to be updated
		uint4 fileAddress;

		// temporarily lock the file while in RAM
		HeapLock(pSymEntry->handle);

		// get the starting address of the allocated file
		fileAddress = HeapDeref (pSymEntry->handle) + 2;

		// copy the contents of the game data to the new file
		memcopy (fileAddress, bufferAddress, bufferSize);

		// move the file to Flash memory
		//EM_moveSymToExtMem (SYMSTR (fileName), HS_NULL);

		// unlock the file
		HeapUnlock(pSymEntry->handle);
	}

	return true;
}

void* SaveFile(
		char* filePath,
		char* fileName,
		char* fileExtension,
		uint4 fileAddress,
		uint2 fileSize)
{
	typedef struct
	{
		uint2 fileSize;
	} StartOfFile_Type;

	typedef struct
	{
		uint1 eofMarker;
		uint1 fileExtension[4];
		uint1 fileTag;
	} EndOfFile_Type;

	SYM_ENTRY* pSymEntry;
	uint4 fileAddressOld;
	uint2 fileSizeOld;
	uint2 allocRequired = 0;

	pSymEntry = DerefSym (SymFind (SYMSTR (MakePath (filePath, fileName))));

	if (pSymEntry)
	{
		// If the file locked, then unlock it
		if (HeapGetLock(pSymEntry->handle))
			HeapUnlock(pSymEntry->handle);

		fileAddressOld = (uint4) HeapDeref (pSymEntry->handle);
		fileSizeOld = *(uint2*)fileAddressOld;

		if (fileSize != fileSizeOld)
		{
			// Since the file exists, lets delete it and start over
			HeapFree (pSymEntry->handle);
			allocRequired = 1;
		}
	}
	else
	{
		// The file does not exist, so create it
		pSymEntry = DerefSym (SymAdd (SYMSTR (MakePath (filePath, fileName))));
	}

	if (pSymEntry)
	{
		pSymEntry->handle = HeapAlloc (fileSize + sizeof(StartOfFile_Type) + sizeof(EndOfFile_Type));

		if (pSymEntry->handle != H_NULL)
		{
//			pTopScores = HeapDeref (pSymEntry->handle);

//			pTopScores->TableSize = TableSize - 2;
//			pTopScores->EndOfDataMarker = 0;
//			sprintf(pTopScores->FileType, "%s", "HSC");
//			pTopScores->FileTag = OTH_TAG;


//			return IK_Success;
		}
	}

	return 0;
}

char* MakePath(
		char* filePath,
		char* fileName)
{
	static char fullPath[18];
	uint2 pathLength;

	pathLength = strlen(filePath);

	if (pathLength > 0)
	{
		if (pathLength > 8)
			pathLength = 8; // this truncates a folder name longer than 8 chars

		strncpy(fullPath, filePath, pathLength);
	}
	else //if (pathLength == 0)
	{
		pathLength = 4;
		strncpy(fullPath, "MAIN", pathLength);
	}

	// add the filename to the path
	strncpy(&fullPath[pathLength], "\\", 1);
	strncpy(&fullPath[pathLength+1], fileName, 8); // this truncates a folder name longer than 8 chars

	return fullPath;
}
*/
/*
int CloseFile(
		char* filePath,
		char* fileName)
{
	SYM_ENTRY* pSymEntry;

	pSymEntry = GetSymEntry(filePath, fileName);
//	pSymEntry = DerefSym (SymFind (SYMSTR (MakePath (filePath, fileName))));

	if (pSymEntry)
	{
		if (HeapGetLock(pSymEntry->handle))
			HeapUnlock(pSymEntry->handle);

		return 0; // Success
	}

	return 1; // Failed
}
*/
